from pybuildkite.users import Users

import pytest


def test_get_current_user(fake_client):
    """
    Test get user
    """
    users = Users(fake_client, "https://api.buildkite.com/v2/")
    users.get_current_user()
    fake_client.get.assert_called_with(users.path)



from posixpath import join as urljoin
from pybuildkite.client import Client


class Artifacts(Client):
    """
    Artifacts operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = urljoin(base_url, "organizations/{}/pipelines/{}/builds/{}/")

    def list_artifacts_for_build(
        self, organization, pipeline, build, page=0, with_pagination=False
    ):
        """
        Returns a paginated list of a build's artifacts across all of its jobs.

        :param organization: organization slug
        :param pipeline: pipeline slug
        :param build: build number
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of a build’s artifacts across all of its jobs.
        """
        url = self.path + "artifacts/"
        query_params = {"page": page}
        return self.client.get(
            url.format(organization, pipeline, build),
            query_params=query_params,
            with_pagination=with_pagination,
        )

    def list_artifacts_for_job(
        self, organization, pipeline, build, job, page=0, with_pagination=False
    ):
        """
        Returns a paginated list of a jobs's artifacts.

        :param organization: organization slug
        :param pipeline: pipeline slug
        :param build: build number
        :param job: job id
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of a job’s artifacts.
        """
        url = self.path + "jobs/{}/artifacts/"
        query_params = {"page": page}
        return self.client.get(
            url.format(organization, pipeline, build, job),
            query_params=query_params,
            with_pagination=with_pagination,
        )

    def get_artifact(self, organization, pipeline, build, job, artifact):
        """
        Returns an artifact.

        :param organization: organization slug
        :param pipeline: pipeline slug
        :param build: build number
        :param job: job id
        :param artifact: artifact id
        :return: Returns an artifact.
        """
        url = self.path + "jobs/{}/artifacts/{}/"
        return self.client.get(url.format(organization, pipeline, build, job, artifact))

    def download_artifact(
        self, organization, pipeline, build, job, artifact, as_stream=False
    ):
        """
        Returns the content of an artifact as bytes.

        With as_stream=True you get an iterator of bytes chunks.

        :param organization: organization slug
        :param pipeline: pipeline slug
        :param build: build number
        :param job: job id
        :param artifact: artifact id
        :param as_stream: stream the artifact content
        :return: Returns the content of an artifact.
        """
        headers = {"Accept": "application/octet-stream"}
        url = self.path + "jobs/{}/artifacts/{}/download/"
        return self.client.get(
            url.format(organization, pipeline, build, job, artifact),
            headers=headers,
            as_stream=as_stream,
        )

    # TODO Delete artifact



import pytest

from pybuildkite.jobs import Jobs, LogFormat


def test_job_logs_can_be_requested_in_a_default_format(fake_client):
    """
    Test job logs can be requested in a default format
    """
    jobs = Jobs(fake_client, "base")
    jobs.get_job_log("org_slug", "pipe_slug", "build_no", 123)
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/log"
    fake_client.get.assert_called_with(url, headers={"Accept": "text/html"})


def test_job_logs_can_be_requested_in_a_user_specified_format(fake_client):
    """
    Test job logs can be requested in a user specified format
    """
    jobs = Jobs(fake_client, "base")
    jobs.get_job_log("org_slug", "pipe_slug", "build_no", 123, "some/thing")
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/log"
    fake_client.get.assert_called_with(url, headers={"Accept": "some/thing"})


def test_job_logs_can_be_requested_in_a_predefined_format(fake_client):
    """
    Test job logs can be requested in a predefined format
    """
    jobs = Jobs(fake_client, "base")
    jobs.get_job_log("org_slug", "pipe_slug", "build_no", 123, LogFormat.TEXT)
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/log"
    fake_client.get.assert_called_with(url, headers={"Accept": "text/plain"})


def test_log_format_can_be_nicely_printed_like_a_string():
    """
    Test that the log can be nicely formatted like a string
    """
    assert str(LogFormat.HTML) == "text/html"


def test_get_job_env(fake_client):
    jobs = Jobs(fake_client, "base")
    jobs.get_job_environment_variables("org_slug", "pipe_slug", "build_no", 123)
    fake_client.get.assert_called_with(
        jobs.path.format("org_slug", "pipe_slug", "build_no", 123) + "/env"
    )


def test_retry_job(fake_client):
    jobs = Jobs(fake_client, "base")
    jobs.retry_job("org_slug", "pipe_slug", "build_no", 123)
    fake_client.put.assert_called_with(
        jobs.path.format("org_slug", "pipe_slug", "build_no", 123) + "/retry"
    )


def test_unblock_job(fake_client):
    jobs = Jobs(fake_client, "base")
    jobs.unblock_job("org_slug", "pipe_slug", "build_no", 123, "name")
    fake_client.put.assert_called_with(
        jobs.path.format("org_slug", "pipe_slug", "build_no", 123) + "/unblock",
        body={"fields": "name", "unblocker": None},
    )


def test_delete_job_log(fake_client):
    jobs = Jobs(fake_client, "base")
    jobs.delete_job_log("org_slug", "pipe_slug", "build_no", 123)
    fake_client.delete.assert_called_with(
        jobs.path.format("org_slug", "pipe_slug", "build_no", 123) + "/log"
    )



from pybuildkite.client import Client


class Annotations(Client):
    """
    Annotation operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = base_url + "organizations/{}/pipelines/{}/builds/{}/annotations/"

    def list_annotations(
        self, organization, pipeline, build, page=0, with_pagination=False
    ):
        """
        Returns a paginated list of the user’s annotations.

        :param organization: organization slug
        :param pipeline: pipeline slug
        :param build: build number
        :return: Returns a paginated list of the user’s annotations.
        """
        query_params = {"page": page}
        return self.client.get(
            self.path.format(organization, pipeline, build),
            query_params=query_params,
            with_pagination=with_pagination,
        )



from pybuildkite.meta import Meta


def test_get_meta_information(fake_client):
    """
    Test get user
    """
    meta = Meta(fake_client, "https://api.buildkite.com/v2/")
    meta.get_meta_information()
    fake_client.get.assert_called_with(meta.path)



import requests
from urllib.parse import urlparse


class Client(object):
    """
    Internal API Client
    """

    def __init__(self, per_page=100):
        """
        Create class
        
        :param per_page: Number of items per page for API requests (default: 100)
        """
        self.access_token = ""
        self.per_page = per_page

    def is_access_token_set(self):
        """
        Has this client got an access token set in it

        :return: true or false
        """
        return not self.access_token == ""

    def set_client_access_token(self, access_token):
        """
        Set an access token to use for API calls
        :param access_token: The token
        """
        self.access_token = access_token

    def request(
        self,
        method,
        url,
        query_params=None,
        body=None,
        headers=None,
        with_pagination=False,
        as_stream=False,
    ):
        """
        Make a request to the API

        The request will be authorised if the access token is set.

        With no accept header set or if set to "application/json",
        the response will be parsed as Json and returned as a dict.
        With any other accept header, the response will be returned as bytes.
        With as_stream=True you will get an iterator of bytes chunks instead.

        :param method: HTTP method to use
        :param url: URL to call
        :param query_params: Query parameters to use
        :param body: Body of the request
        :param headers: Dictionary of headers to use in HTTP request
        :param with_pagination: Bool to return a response with pagination attributes
        :param as_stream: Bool to stream the bytes response
        :return: response return as parsed json, bytes or bytes chunks iterator
        """
        if headers is None:
            headers = {}

        if self.access_token:
            headers["Authorization"] = "Bearer {}".format(self.access_token)

        if body:
            body = self._clean_query_params(body)

        query_params = self._clean_query_params(query_params or {})
        query_params["per_page"] = str(self.per_page)

        query_params = self._convert_query_params_to_string_for_bytes(query_params)
        response = requests.request(
            method,
            url,
            headers=headers,
            params=str.encode(query_params),
            json=body,
            stream=as_stream,
        )

        response.raise_for_status()

        if with_pagination:
            response = self._get_paginated_response(response)
            return response
        if (
            method == "DELETE"
            or response.status_code == 204
            or response.headers.get("content-type") is None
        ):
            return response.ok
        if headers.get("Accept") is None or headers.get("Accept") == "application/json":
            return response.json()
        elif as_stream:
            return response.iter_content(chunk_size=None, decode_unicode=False)
        else:
            return response.content

    def _get_paginated_response(self, response):
        """
        Return a Response object with pagination data

        :return: Response object
        """
        response_object = Response(response.json())
        response_object.append_pagination_data(response.headers)
        return response_object

    def get(
        self, url, query_params=None, headers=None, with_pagination=False, as_stream=False
    ):
        """
        Make a GET request to the API

        The request will be authorised if the access token is set

        :param url: URL to call
        :param query_params: Query parameters to append to URL
        :param headers: Dictionary of headers to use in HTTP request
        :param with_pagination: Bool to return a response with pagination attributes
        :param as_stream: Bool to stream the response
        :return: If headers are set response text is returned, otherwise parsed response is returned
        """
        return self.request(
            "GET",
            url=url,
            query_params=query_params,
            headers=headers,
            with_pagination=with_pagination,
            as_stream=as_stream,
        )

    def post(self, url, body=None, headers=None, query_params=None):
        """
        Make a POST request to the API

        The request will be authorised if the access token is set

        :param url: URL to call
        :param body: Body of the request
        :param query_params: Query parameters to append to URL
        :param headers: Dictionary of headers to use in HTTP request
        :return: If headers are set response text is returned, otherwise parsed response is returned
        """

        return self.request(
            "POST", url=url, query_params=query_params, body=body, headers=headers
        )

    def put(self, url, body=None, headers=None, query_params=None):
        """
        Make a PUT request to the API

        The request will be authorised if the access token is set

        :param url: URL to call
        :param body: Body of the request
        :param headers: Dictionary of headers to use in HTTP request
        :param query_params: Query parameters to append to URL
        :return: If headers are set response text is returned, otherwise parsed response is returned
        """

        return self.request(
            "PUT", url=url, query_params=query_params, body=body, headers=headers
        )

    def delete(self, url, body=None, headers=None, query_params=None):
        """
        Make a DELETE request to the API

        The request will be authorised if the access token is set

        :param url: URL to call
        :param body: Body of the request
        :param query_params: Query parameters to append to URL
        :param headers: Dictionary of headers to use in HTTP request
        :return: If headers are set response text is returned, otherwise parsed response is returned
        """

        return self.request(
            "DELETE", url=url, query_params=query_params, body=body, headers=headers
        )

    def patch(self, url, body=None, headers=None, query_params=None):
        """
        Make a PATCH request to the API

        The request will be authorised if the access token is set

        :param url: URL to call
        :param body: Body of the request
        :param query_params: Query parameters to append to URL
        :param headers: Dictionary of headers to use in HTTP request
        :return: If headers are set response text is returned, otherwise parsed response is returned
        """
        return self.request(
            "PATCH", url=url, query_params=query_params, body=body, headers=headers
        )

    @staticmethod
    def _clean_query_params(query_params):
        """

        :param query_params:
        :return:
        """
        return {key: value for key, value in query_params.items() if value is not None}

    @staticmethod
    def _convert_query_params_to_string_for_bytes(query_params):
        """
        Required to set multiple build states i.e ?state[]=running&state[]=scheduled

        :param query_params: query parameters
        :return: bytes of query param
        """
        query_string = ""
        for key, value in query_params.items():
            if query_string != "":
                query_string += "&"
            if key == "state":
                query_string += value
            elif key == "branch":
                query_string += value
            else:
                query_string += key + "=" + str(value)
        return query_string


class Response:
    """
    Response object used for pagination requests
    """

    def __init__(self, body):
        self.body = body
        self.next_page = None
        self.last_page = None
        self.first_page = None
        self.previous_page = None

    def append_pagination_data(self, response_headers):
        """
        Add pagination data to the response based on response headers

        :param respone_headers: dict containing pagination information from the API
        """
        if "Link" in response_headers and len(response_headers["Link"]) > 0:
            for link in response_headers["Link"].split(", "):
                url, page_value = link.split("; ", 1)
                if page_value == 'rel="next"':
                    self.next_page = self._get_page_number_from_url(url)
                elif page_value == 'rel="last"':
                    self.last_page = self._get_page_number_from_url(url)
                elif page_value == 'rel="first"':
                    self.first_page = self._get_page_number_from_url(url)
                elif page_value == 'rel="prev"':
                    self.previous_page = self._get_page_number_from_url(url)
        return self

    def _get_page_number_from_url(self, url):
        """
        Gets the page number for pagination from a given url

        :param url: url to retrieve page from
        :return: int of page in url
        """
        parsed = urlparse(url)
        query_string = parsed.query
        for segment in query_string.split("&"):
            key, value = segment.split("=", 1)
            if key == "page":
                return int(value)
        return 0



import pytest
from pybuildkite.buildkite import (
    Buildkite,
    Pipelines,
    Agents,
    Builds,
    Jobs,
    Emojis,
    Annotations,
    Artifacts,
    Teams,
    Users,
    Organizations,
    Meta,
)
from pybuildkite.exceptions import NoAcccessTokenException


def test_access_token_not_set_raises_exception():
    """
    Test that exception is raised if no access token is set
    """
    with pytest.raises(NoAcccessTokenException):
        buildkite = Buildkite()
        buildkite.agents()


def test_access_token_set():
    """
    Test that methods can be called when access token is set
    """
    buildkite = Buildkite()
    buildkite.set_access_token("FAKE-ACCESS-TOKEN")
    assert buildkite.agents()


def test_buildkite_with_custom_per_page():
    """
    Test that Buildkite class accepts and uses custom per_page parameter
    """
    buildkite = Buildkite(per_page=50)
    buildkite.set_access_token("FAKE-ACCESS-TOKEN")
    assert buildkite.client.per_page == 50
    assert buildkite.agents()


def test_buildkite_default_per_page():
    """
    Test that Buildkite class uses default per_page value
    """
    buildkite = Buildkite()
    assert buildkite.client.per_page == 100


@pytest.mark.parametrize(
    "function, expected_type",
    [
        (Buildkite().pipelines, Pipelines),
        (Buildkite().builds, Builds),
        (Buildkite().jobs, Jobs),
        (Buildkite().agents, Agents),
        (Buildkite().emojis, Emojis),
        (Buildkite().artifacts, Artifacts),
        (Buildkite().teams, Teams),
        (Buildkite().users, Users),
        (Buildkite().annotations, Annotations),
        (Buildkite().organizations, Organizations),
        (Buildkite().meta, Meta),
    ],
)
def test_eval(function, expected_type):
    # buildkite = Buildkite()
    # buildkite.set_access_token("FAKE-ACCESS-TOKEN")
    function.__self__.set_access_token("FAKE-ACCESS-TOKEN")
    pipelines = function()
    assert isinstance(pipelines, expected_type)



from pybuildkite.organizations import Organizations

import pytest


def test_Organizations(fake_client):
    """
    Test organization classes instances
    """
    org = Organizations(fake_client, "https://api.buildkite.com/v2/")
    assert org.client == fake_client
    assert org.path == "https://api.buildkite.com/v2/organizations/"


def test_list_all(fake_client):
    """pye
    Test organization class 'list_all()'  Method
    """
    org = Organizations(fake_client, "https://api.buildkite.com/v2/")
    org.list_all()
    fake_client.get.assert_called_with(
        org.path, query_params={"page": 0}, with_pagination=False
    )


def test_list_all_with_pagination(fake_client):
    """
    Test organization class 'list_all()'  Method with pagination
    """
    org = Organizations(fake_client, "https://api.buildkite.com/v2/")
    org.list_all(page=2, with_pagination=True)
    fake_client.get.assert_called_with(
        org.path, query_params={"page": 2}, with_pagination=True
    )


def test_get_org(fake_client):
    """
    Test organization class 'get_org()' method
    """
    org = Organizations(fake_client, "https://api.buildkite.com/v2/")
    org.get_org("Test_org")
    fake_client.get.assert_called_with(org.path + "Test_org")



from pybuildkite.client import Client
from pybuildkite.organizations import Organizations
from pybuildkite.pipelines import Pipelines
from pybuildkite.builds import Builds, BuildState
from pybuildkite.jobs import Jobs, LogFormat
from pybuildkite.agents import Agents
from pybuildkite.emojis import Emojis
from pybuildkite.annotations import Annotations
from pybuildkite.artifacts import Artifacts
from pybuildkite.teams import Teams
from pybuildkite.users import Users
from pybuildkite.access_tokens import AccessTokens
from pybuildkite.meta import Meta
from pybuildkite.decorators import requires_token


class Buildkite(object):
    """
    Public API for Buildkite
    """

    def __init__(self, per_page=100):
        """
        Create a new client
        
        :param per_page: Number of items per page for API requests (default: 100)
        """
        self.client = Client(per_page)
        self.base_url = "https://api.buildkite.com/v2/"

    def set_access_token(self, access_token):
        """
        Set the access token to be used to authenticate the requests
        :param access_token: The access token
        """
        self.client.set_client_access_token(access_token)

    @requires_token
    def organizations(self):
        """
        Get Organization operations for the Buildkite API

        :return: Client
        """
        return Organizations(self.client, self.base_url)

    @requires_token
    def pipelines(self):
        """
        Get Pipeline operations for the Buildkite API

        :return: Client
        """
        return Pipelines(self.client, self.base_url)

    @requires_token
    def builds(self):
        """
        Get Build operations for the Buildkite API

        :return: Client
        """
        return Builds(self.client, self.base_url)

    @requires_token
    def jobs(self):
        """
        Get Job operations for the Buildkite API

        :return: Client
        """
        return Jobs(self.client, self.base_url)

    @requires_token
    def agents(self):
        """
        Get Agent operations for the Buildkite API

        :return: Client
        """
        return Agents(self.client, self.base_url)

    @requires_token
    def emojis(self):
        """
        Get Emoji operations for the Buildkite API
        """
        return Emojis(self.client, self.base_url)

    @requires_token
    def annotations(self):
        """
        Get Annotation operations for the Buildkite API
        """
        return Annotations(self.client, self.base_url)

    @requires_token
    def artifacts(self):
        """
        Get Artifact operations for the Buildkite API
        """
        return Artifacts(self.client, self.base_url)

    @requires_token
    def teams(self):
        """
        Get Team operations for the Buildkite API
        """
        return Teams(self.client, self.base_url)

    @requires_token
    def users(self):
        """
        Get User operations for the Buildkite API
        """
        return Users(self.client, self.base_url)

    @requires_token
    def access_tokens(self):
        """
        Get Access Token operations for the Buildkite API

        :return: Client
        """
        return AccessTokens(self.client, self.base_url)

    def meta(self):
        """
        Get Meta operations for the Buildkite API

        :return: Client
        """
        return Meta(self.client, self.base_url)



from pybuildkite.pipelines import PipelineException, Pipelines

import pytest


def test_Pipelines(fake_client):
    """
    Test organization classes instances
    """
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    assert pipeline.client == fake_client
    assert pipeline.path == "https://api.buildkite.com/v2/organizations/{}/pipelines/"


def test_list_pipelines(fake_client):
    """
    Test organization class 'list_pipelines()'  Method
    """
    pipelines = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipelines.list_pipelines("Test_org")
    fake_client.get.assert_called_with(
        pipelines.path.format("Test_org"),
        query_params={"page": 0},
        with_pagination=False,
    )


def test_get_pipeline(fake_client):
    """
    Test organization class 'get_pipeline()' method
    """
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.get_pipeline("Test_org", "Test_pipeline")
    fake_client.get.assert_called_with(
        pipeline.path.format("Test_org") + "Test_pipeline"
    )


def test_create_pipeline(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_pipeline("test_org", "test_pipeline", "my_repo")
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "steps": [
                {
                    "type": "script",
                    "name": ":pipeline:",
                    "command": "buildkite-agent pipeline upload",
                }
            ],
            "team_uuids": None,
            "branch_configuration": None,
            "default_branch": None,
            "provider_settings": None,
        },
    )


def test_create_pipeline_with_teams(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_pipeline("test_org", "test_pipeline", "my_repo", team_uuids=["123"])
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "steps": [
                {
                    "type": "script",
                    "name": ":pipeline:",
                    "command": "buildkite-agent pipeline upload",
                }
            ],
            "team_uuids": ["123"],
            "branch_configuration": None,
            "default_branch": None,
            "provider_settings": None,
        },
    )


def test_create_pipeline_with_branch_configuration(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_pipeline(
        "test_org", "test_pipeline", "my_repo", branch_configuration="v*"
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "steps": [
                {
                    "type": "script",
                    "name": ":pipeline:",
                    "command": "buildkite-agent pipeline upload",
                }
            ],
            "team_uuids": None,
            "branch_configuration": "v*",
            "default_branch": None,
            "provider_settings": None,
        },
    )


def test_create_pipeline_with_default_branch(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_pipeline(
        "test_org", "test_pipeline", "my_repo", default_branch="main"
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "steps": [
                {
                    "type": "script",
                    "name": ":pipeline:",
                    "command": "buildkite-agent pipeline upload",
                }
            ],
            "team_uuids": None,
            "branch_configuration": None,
            "default_branch": "main",
            "provider_settings": None,
        },
    )


def test_create_pipeline_with_provider_settings(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_pipeline(
        "test_org", "test_pipeline", "my_repo", provider_settings={"build_tags": True}
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "steps": [
                {
                    "type": "script",
                    "name": ":pipeline:",
                    "command": "buildkite-agent pipeline upload",
                }
            ],
            "team_uuids": None,
            "branch_configuration": None,
            "default_branch": None,
            "provider_settings": {"build_tags": True},
        },
    )


def test_create_yaml_pipeline(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_yaml_pipeline(
        "test_org", "test_pipeline", "my_repo", "steps:\n  - command: ls"
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "configuration": "steps:\n  - command: ls",
            "team_uuids": None,
            "branch_configuration": None,
            "default_branch": None,
            "provider_settings": None,
        },
    )


def test_create_yaml_pipeline_with_teams(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_yaml_pipeline(
        "test_org",
        "test_pipeline",
        "my_repo",
        "steps:\n  - command: ls",
        team_uuids=["123"],
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "configuration": "steps:\n  - command: ls",
            "team_uuids": ["123"],
            "branch_configuration": None,
            "default_branch": None,
            "provider_settings": None,
        },
    )


def test_create_yaml_pipeline_with_branch_configuration(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_yaml_pipeline(
        "test_org",
        "test_pipeline",
        "my_repo",
        "steps:\n  - command: ls",
        branch_configuration="v*",
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "configuration": "steps:\n  - command: ls",
            "team_uuids": None,
            "branch_configuration": "v*",
            "default_branch": None,
            "provider_settings": None,
        },
    )


def test_create_yaml_pipeline_with_default_branch(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_yaml_pipeline(
        "test_org",
        "test_pipeline",
        "my_repo",
        "steps:\n  - command: ls",
        default_branch="main",
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "configuration": "steps:\n  - command: ls",
            "team_uuids": None,
            "branch_configuration": None,
            "default_branch": "main",
            "provider_settings": None,
        },
    )


def test_create_yaml_pipeline_with_provider_settings(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.create_yaml_pipeline(
        "test_org",
        "test_pipeline",
        "my_repo",
        "steps:\n  - command: ls",
        provider_settings={"build_tags": True},
    )
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org"),
        body={
            "name": "test_pipeline",
            "repository": "my_repo",
            "configuration": "steps:\n  - command: ls",
            "team_uuids": None,
            "branch_configuration": None,
            "default_branch": None,
            "provider_settings": {"build_tags": True},
        },
    )


def test_delete_pipeline(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.delete_pipeline("test_org", "test_pipeline")
    fake_client.delete.assert_called_with(
        pipeline.path.format("test_org") + "test_pipeline"
    )


def test_update_pipeline(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.update_pipeline(
        "test_org",
        "test_pipeline",
        name="Name Change Test",
        env={"TEST_ENV_VAR": "VALUE"},
        skip_queued_branch_builds=True,
    )
    fake_client.patch.assert_called_with(
        pipeline.path.format("test_org") + "test_pipeline",
        body={
            "branch_configuration": None,
            "cancel_running_branch_builds": None,
            "cancel_running_branch_builds_filter": None,
            "default_branch": None,
            "description": None,
            "env": {"TEST_ENV_VAR": "VALUE"},
            "name": "Name Change Test",
            "provider_settings": None,
            "repository": None,
            "steps": None,
            "configuration": None,
            "skip_queued_branch_builds": True,
            "skip_queued_branch_builds_filter": None,
            "visibility": None,
        },
    )


def test_update_pipeline_configuration_and_steps(fake_client):
    with pytest.raises(PipelineException):
        pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
        pipeline.update_pipeline(
            "test_org",
            "test_pipeline",
            name="Name Change Test",
            env={"TEST_ENV_VAR": "VALUE"},
            configuration="",
            steps={},
        )


def test_archive_pipeline(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.archive_pipeline("test_org", "test_pipeline")
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org") + "test_pipeline" + "/archive"
    )


def test_unarchive_pipeline(fake_client):
    pipeline = Pipelines(fake_client, "https://api.buildkite.com/v2/")
    pipeline.unarchive_pipeline("test_org", "test_pipeline")
    fake_client.post.assert_called_with(
        pipeline.path.format("test_org") + "test_pipeline" + "/unarchive"
    )



from pybuildkite.teams import Teams

import pytest


def test_teams(fake_client):
    """
    Test team initialization
    """
    teams = Teams(fake_client, "https://api.buildkite.com/v2/")
    assert teams.client == fake_client
    assert teams.path == "https://api.buildkite.com/v2/organizations/{}/teams"


def test_list_teams_with_no_user_id(fake_client):
    """
    Test get teams
    """
    teams = Teams(fake_client, "https://api.buildkite.com/v2/")
    teams.list_teams("test_org")
    fake_client.get.assert_called_with(
        teams.path.format("test_org"),
        query_params={"page": 0, "user_id": None},
        with_pagination=False,
    )


def test_list_teams_with_user_id(fake_client):
    """
    Test get teams
    """
    teams = Teams(fake_client, "https://api.buildkite.com/v2/")
    teams.list_teams(organization="test_org", user_id="test_user")
    fake_client.get.assert_called_with(
        teams.path.format("test_org", "test_user"),
        query_params={"page": 0, "user_id": "test_user"},
        with_pagination=False,
    )



import datetime

import pytest

from pybuildkite.builds import Builds, BuildState
from pybuildkite.exceptions import (
    BuildStateNotAList,
    NotValidBuildState,
    NotValidDateTime,
)


def test_list_all_builds_single_build_state(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.list_all(states=[BuildState.RUNNING])
    fake_client.get.assert_called_with(
        builds.path_for_all,
        {
            "creator": None,
            "created_from": None,
            "created_to": None,
            "finished_from": None,
            "state": "state=running",
            "branch": None,
            "commit": None,
            "include_retried_jobs": None,
            "page": 0,
        },
        with_pagination=False,
    )


def test_list_all_builds_multiple_build_states(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.list_all(states=[BuildState.RUNNING, BuildState.FINISHED])
    fake_client.get.assert_called_with(
        builds.path_for_all,
        {
            "creator": None,
            "created_from": None,
            "created_to": None,
            "finished_from": None,
            "state": "state[]=running&state[]=finished",
            "branch": None,
            "commit": None,
            "include_retried_jobs": None,
            "page": 0,
        },
        with_pagination=False,
    )


def test_build_state_must_be_list(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    with pytest.raises(BuildStateNotAList):
        builds.list_all(states=BuildState.RUNNING)


def test_build_state_must_be_valid(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    with pytest.raises(NotValidBuildState):
        builds.list_all(states=["ran"])


def test_date_must_be_valid(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    with pytest.raises(NotValidDateTime):
        builds.list_all(created_from="2017-02-03")


def test_correct_date_formats(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")

    check_correct_dates(builds.list_all_for_org, fake_client, organization="org")
    check_correct_dates(builds.list_all, fake_client)
    check_correct_dates(
        builds.list_all_for_pipeline, fake_client, organization="org", pipeline="pipe"
    )


def check_correct_dates(func, fake_client, **kwargs):
    func(
        **kwargs,
        created_to=datetime.date(2020, 1, 2),
        created_from=datetime.datetime(2020, 1, 1, 10, 10, 10),
    )
    args = fake_client.get.call_args[0][1]
    assert args["created_from"] == "2020-01-01T10:10:10"
    assert args["created_to"] == "2020-01-02"
    assert args["finished_from"] == None


def test_list_all_builds_for_org(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.list_all_for_org("org_slug")
    fake_client.get.assert_called_with(
        builds.path_by_org.format("org_slug"),
        {
            "creator": None,
            "created_from": None,
            "created_to": None,
            "finished_from": None,
            "state": None,
            "branch": None,
            "commit": None,
            "include_retried_jobs": None,
            "page": 0,
        },
        with_pagination=False,
    )


def test_list_all_for_pipeline(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.list_all_for_pipeline("org_slug", "pipeline_id")
    fake_client.get.assert_called_with(
        builds.path_by_pipeline.format("org_slug", "pipeline_id"),
        {
            "creator": None,
            "created_from": None,
            "created_to": None,
            "finished_from": None,
            "state": None,
            "branch": None,
            "commit": None,
            "include_retried_jobs": None,
            "page": 0,
        },
        with_pagination=False,
    )


def test_get_build_by_number(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.get_build_by_number("org_slug", "pipeline_id", "build_number")
    fake_client.get.assert_called_with(
        builds.path_for_build_number.format("org_slug", "pipeline_id", "build_number"),
        query_params={"include_retried_jobs": None},
    )


def test_get_build_by_number_with_retried_jobs(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.get_build_by_number(
        "org_slug", "pipeline_id", "build_number", include_retried_jobs=True
    )
    fake_client.get.assert_called_with(
        builds.path_for_build_number.format("org_slug", "pipeline_id", "build_number"),
        query_params={"include_retried_jobs": True},
    )


def test_meta_data_url(fake_client):
    """
    Verifies if url is created properly when using meta_data
    """
    meta_data = {"key1": 1, "key2": "2"}
    builds = Builds(fake_client, "base")
    builds.list_all(meta_data=meta_data)
    name, args, kwargs = fake_client.method_calls[-1]
    _, query_params = args
    assert query_params["meta_data[key1]"] == 1
    assert query_params["meta_data[key2]"] == "2"


def test_no_meta_data_url(fake_client):
    """
    Verifies if url is created properly when using meta_data
    """
    builds = Builds(fake_client, "base")
    builds.list_all()

    name, args, kwargs = fake_client.method_calls[-1]
    _, query_params = args
    for key in query_params:
        assert "meta_data" not in key


def test_create_build(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.create_build("org_slug", "pipeline_id", "COMMITSHA", "branch_name")
    fake_client.post.assert_called_with(
        builds.path_by_pipeline.format("org_slug", "pipeline_id"),
        {
            "commit": "COMMITSHA",
            "branch": "branch_name",
            "author": None,
            "clean_checkout": None,
            "env": None,
            "ignore_pipeline_branch_filters": None,
            "message": None,
            "meta_data": None,
            "pull_request_base_branch": None,
            "pull_request_id": None,
            "pull_request_repository": None,
            "pull_request_labels": None,
        },
    )


def test_cancel_build(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.cancel_build("org_slug", "pipeline_id", "build_number")
    fake_client.put.assert_called_with(
        builds.path_for_build_number.format("org_slug", "pipeline_id", "build_number")
        + "/cancel"
    )


def test_rebuild_build(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.rebuild_build("org_slug", "pipeline_id", "build_number")
    fake_client.put.assert_called_with(
        builds.path_for_build_number.format("org_slug", "pipeline_id", "build_number")
        + "/rebuild"
    )


def test_multi_branch(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.rebuild_build("org_slug", "pipeline_id", "build_number")

    builds.list_all_for_pipeline(
        organization="org", pipeline="pipe", branch=["main", "master"]
    )

    args = fake_client.get.call_args[0][1]
    assert args["branch"] == "branch[]=main&branch[]=master"


def test_single_branch(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    builds.rebuild_build("org_slug", "pipeline_id", "build_number")

    builds.list_all_for_pipeline(organization="org", pipeline="pipe", branch="main")

    args = fake_client.get.call_args[0][1]
    assert args["branch"] == "branch=main"


def test_branch_with_special_characters(fake_client):
    builds = Builds(fake_client, "https://api.buildkite.com/v2/")
    branch_name = "feature#123"
    builds.list_all_for_pipeline(organization="org", pipeline="pipe", branch=branch_name)
    args = fake_client.get.call_args[0][1]
    # '#' should be encoded as '%23'
    assert args["branch"] == "branch=feature%23123"



from typing import Callable
from pybuildkite.exceptions import NoAcccessTokenException


def requires_token(func: Callable):
    """
    This annotation protects API calls that require authentication.

    It will cause them to raise NoAcccessTokenException if the token is not set in the client.

    :return: Function decorated with the protection
    """

    def wrapper(self, *args, **kwargs):
        """
        Call func or raise NoAcccessTokenException if no access token is set

        :param self:
        :param args: Optional
        :param kwargs: Optional

        :raises NoAcccessTokenException: If no access token is set
        :return:
        """
        if not self.client.is_access_token_set():
            raise NoAcccessTokenException
        else:
            return func(self, *args, **kwargs)

    return wrapper



from unittest.mock import patch

import pytest

from pybuildkite.client import Client, Response


class TestClient:
    """
    Test functionality of the client class
    """

    def test_access_token_setting(self):
        """
        Test functionality of is_acces_token_set
        """
        client = Client()
        assert not client.is_access_token_set()
        client.set_client_access_token("FAKE-TOKEN")
        assert client.is_access_token_set()

    def test_per_page_parameter(self):
        """
        Test that per_page parameter is properly set and used
        """
        client = Client(per_page=50)
        assert client.per_page == 50
        
        # Test default value
        client_default = Client()
        assert client_default.per_page == 100

    def test_request_with_custom_per_page(self):
        """
        Test that custom per_page value is used in requests
        """
        fake_client = Client(per_page=25)

        with patch("requests.request") as request:
            request.return_value.json.return_value = {}

            fake_client.request("GET", "http://www.google.com/")

        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={},
            json=None,
            params=b"per_page=25",
            stream=False,
        )

    def test_clean_query_params(self):
        """
        Test that params with None are cleaned
        """
        original_query_params = {
            "name": "FAKE-NAME",
            "hostname": "FAKE-HOSTNAME",
            "version": None,
        }

        cleaned_query_params = {"name": "FAKE-NAME", "hostname": "FAKE-HOSTNAME"}

        client = Client()
        original_query_params = client._clean_query_params(original_query_params)
        assert original_query_params == cleaned_query_params


class TestResponse:
    @pytest.mark.parametrize(
        "url,expected_output",
        [
            ("https://api.buildkite.com/v2/organization/builds?page=2&commit=SHA", 2),
            ("https://api.buildkite.com/v2/page/builds?page=5&commit=SHA", 5),
            ("https://api.buildkite.com/v2/page/builds?commit=SHA", 0),
        ],
    )
    def test_getting_page_from_url(self, url, expected_output):
        response = Response({"Body": "FakeBody"})
        output = response._get_page_number_from_url(url)
        assert output == expected_output

    def test_no_data_added_without_pagination_headers(self):
        response = Response({"Body": "FakeBody"})
        response_headers = ""
        response.append_pagination_data(response_headers)
        assert response.body == {"Body": "FakeBody"}
        assert response.first_page == None
        assert response.last_page == None
        assert response.previous_page == None
        assert response.next_page == None

    def test_pagination_headers_with_next_and_last(self):
        response = Response({"Body": "FakeBody"})
        response_headers = {
            "Date": "Wed, 20 Nov 2019 03:13:27 GMT",
            "Content-Type": "application/json; charset=utf-8",
            "Connection": "keep-alive",
            "Server": "nginx",
            "Link": '<https://api.buildkite.com/v2/builds?access_token=FakeToken&page=2&per_page=100>; rel="next", <https://api.buildkite.com/v2/builds?access_token=FakeToken&page=8&per_page=100>; rel="last"',
            "X-OAuth-Scopes": "read_agents",
        }
        response.append_pagination_data(response_headers)
        assert response.body == {"Body": "FakeBody"}
        assert response.first_page == None
        assert response.last_page == 8
        assert response.previous_page == None
        assert response.next_page == 2


class TestClientRequest:
    """
    Test the request-method of the client class
    """

    def test_request_should_not_include_token_when_not_set(self):
        """
        Test that the access token is not included in the call to
        requests if it isn't actually set.
        """
        fake_client = Client()

        with patch("requests.request") as request:
            request.return_value.json.return_value = {}

            fake_client.request("GET", "http://www.google.com/")

        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={},
            json=None,
            params=b"per_page=100",
            stream=False,
        )

    def test_request_should_return_bytes_when_non_json_accept_header_provided(self):
        """
        Test that the accept header is preserved and that bytes are returned
        """
        fake_client = Client()

        with patch("requests.request") as request:
            request.return_value.content = b"response text"

            resp = fake_client.request(
                "GET",
                "http://www.google.com/",
                headers={"Accept": "application/fake_encoding"},
            )

        expected_params = b"per_page=100"
        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={"Accept": "application/fake_encoding"},
            json=None,
            params=expected_params,
            stream=False,
        )

        assert resp == b"response text"

    def test_request_should_return_byte_stream_when_requested(self):
        """
        Test that the requests response.iter_content(None, False) is returned
        when stream is requested
        """
        fake_client = Client()

        with patch("requests.request") as request:
            request.return_value.iter_content.return_value = [
                b"response",
                b" ",
                b"text",
            ]

            resp = fake_client.request(
                "GET",
                "http://www.google.com/",
                headers={"Accept": "application/fake_encoding"},
                as_stream=True,
            )

        expected_params = b"per_page=100"
        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={"Accept": "application/fake_encoding"},
            json=None,
            params=expected_params,
            stream=True,
        )
        request.return_value.iter_content.assert_called_once_with(
            chunk_size=None, decode_unicode=False
        )

        assert resp == [b"response", b" ", b"text"]

    def test_request_should_return_json_when_json_accept_header_provided(self):
        """
        Test that the requests response.json() decoding is returned when
        no accept header is provided
        """
        fake_client = Client()

        with patch("requests.request") as request:
            request.return_value.json.return_value = {"key": "value"}

            resp = fake_client.request(
                "GET", "http://www.google.com/", headers={"Accept": "application/json"}
            )

        expected_params = b"per_page=100"
        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={"Accept": "application/json"},
            json=None,
            params=expected_params,
            stream=False,
        )

        assert resp == {"key": "value"}

    def test_request_should_return_json_when_no_accept_header_provided(self):
        """
        Test that the requests response.json() decoding is returned when
        no accept header is provided
        """
        fake_client = Client()

        with patch("requests.request") as request:
            request.return_value.json.return_value = {"key": "value"}

            resp = fake_client.request("GET", "http://www.google.com/")

        expected_params = b"per_page=100"
        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={},
            json=None,
            params=expected_params,
            stream=False,
        )

        assert resp == {"key": "value"}

    def test_request_should_include_token_when_set(self):
        """
        Test that the access token is not included in the call to
        requests if it isn't actually set.
        """
        fake_client = Client()
        fake_client.set_client_access_token("ABCDEF1234")

        with patch("requests.request") as request:
            request.return_value.json.return_value = {"key": "value"}

            resp = fake_client.request("GET", "http://www.google.com/")

        expected_params = b"per_page=100"
        request.assert_called_once_with(
            "GET",
            "http://www.google.com/",
            headers={"Authorization": "Bearer ABCDEF1234"},
            json=None,
            params=expected_params,
            stream=False,
        )

        assert resp == {"key": "value"}



import pytest

from pybuildkite.annotations import Annotations


def test_list_annotations(fake_client):
    annotations = Annotations(fake_client, "https://api.buildkite.com/v2/")
    annotations.list_annotations("org_slug", "pipeline_id", "build_number")
    fake_client.get.assert_called_with(
        annotations.path.format("org_slug", "pipeline_id", "build_number"),
        query_params={"page": 0},
        with_pagination=False,
    )



class NoAcccessTokenException(Exception):
    """
    Indicates that an access token was not set when it was required
    """

    pass


class NotValidDateTime(Exception):
    """
    Raised when date is not a valid datetime.date
    """

    pass


class NotValidBuildState(Exception):
    """
    Raised when state is not a valid BuildState
    """

    pass


class BuildStateNotAList(Exception):
    """
    Raised when build state is not passed as a list
    """

    pass



from pybuildkite.client import Client


class Teams(Client):
    """
    Teams operations for the Buildkite API
    """

    def __init__(self, client: Client, base_url: str) -> None:
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = base_url + "organizations/{}/teams"

    def list_teams(
        self,
        organization: str,
        user_id: str = None,
        page: int = 0,
        with_pagination: bool = False,
    ):
        """
        Returns a list of all the teams for a given organization

        :param organization: organization slug
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: bool to return a response with pagination attributes
        :return: Returns a list of all the teams for a given organization
        """
        query_parms = {"page": page, "user_id": user_id}
        return self.client.get(
            self.path.format(organization),
            query_params=query_parms,
            with_pagination=with_pagination,
        )



import pytest

from pybuildkite.emojis import Emojis


def test_list_emojis(fake_client):
    emojis = Emojis(fake_client, "https://api.buildkite.com/v2/")
    emojis.list_emojis("org_slug")
    fake_client.get.assert_called_with(
        emojis.path.format("org_slug"), query_params={"page": 0}, with_pagination=False
    )



from posixpath import join as urljoin
from pybuildkite.client import Client


class Agents(Client):
    """
    Agent operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = urljoin(base_url, "organizations/{}/agents/")

    def list_all(
        self,
        organization,
        name=None,
        hostname=None,
        version=None,
        page=0,
        with_pagination=False,
    ):
        """
        Returns a paginated list of an organization’s agents.

        :param organization: Organization slug
        :param name: Filters the results by the given agent name
        :param hostname: Filters the results by the given hostname
        :param version: Filters the results by the given exact version number
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of an organization’s agents
        """
        query_params = {
            "name": name,
            "hostname": hostname,
            "version": version,
            "page": page,
        }
        return self.client.get(
            self.path.format(organization),
            query_params,
            with_pagination=with_pagination,
        )

    def get_agent(self, organization, agent_id):
        """
        Get an agent

        :param organization: Organization slug
        :param agent_id: Agent id
        :return: Single agent
        """
        return self.client.get(self.path.format(organization) + agent_id)

    def stop_agent(self, organization, agent_id, force=True):
        """
        Stop an agent

        :param organization: Organization slug
        :param agent_id: Agent id
        :param force: Whether or not to force the agent to stop if processing a job
        :return: no content
        """
        body = {"force": force}
        stop = "/stop"
        return self.client.put(
            self.path.format(organization) + agent_id + stop, body=body
        )



from posixpath import join as urljoin
from pybuildkite.client import Client


class AccessTokens(Client):
    """
    Access Token operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = urljoin(base_url, "access-token")

    def get_token(self):
        """
        Get data on the token used for the request
        """
        return self.client.get(self.path)

    def revoke_token(self):
        """
        Revoke the token that is used for the request.
        """
        return self.client.delete(self.path)



import pytest
from pybuildkite.access_tokens import AccessTokens


def test_get_access_token(fake_client):
    """
    Test Get Access Token
    """
    artifacts = AccessTokens(fake_client, "base")
    artifacts.get_token()
    url = "base/access-token"
    fake_client.get.assert_called_with(url)


def test_revoke_access_token(fake_client):
    """
    Test Get Access Token
    """
    artifacts = AccessTokens(fake_client, "base")
    artifacts.revoke_token()
    url = "base/access-token"
    fake_client.delete.assert_called_with(url)



import datetime
from enum import Enum
from typing import List
import urllib.parse

from pybuildkite.client import Client
from pybuildkite.exceptions import (
    BuildStateNotAList,
    NotValidBuildState,
    NotValidDateTime,
)


class BuildState(Enum):
    """
    Valid build states

    The finished state is a shortcut to automatically search for builds with passed, failed, blocked, canceled states.
    """

    RUNNING = "running"
    SCHEDULED = "scheduled"
    PASSED = "passed"
    FAILING = "failing"
    FAILED = "failed"
    BLOCKED = "blocked"
    CANCELED = "canceled"
    CANCELING = "canceling"
    SKIPPED = "skipped"
    NOT_RUN = "not_run"
    FINISHED = "finished"


# TODO needed?
class BuildQueryParams(Enum):
    """
    Query parameters for listing builds
    """

    CREATOR = "creator"
    CREATED_FROM = "created_from"
    CREATED_TO = "created_to"
    FINISHED_FROM = "finished_from"
    STATE = "state"
    META_DATA = "meta_data"
    BRANCH = "branch"
    COMMIT = "commit"


class Builds(Client):
    """
    Build operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path_for_all = base_url + "builds"
        self.path_by_org = base_url + "organizations/{}/builds"
        self.path_by_pipeline = base_url + "organizations/{}/pipelines/{}/builds"
        self.path_for_build_number = (
            base_url + "organizations/{}/pipelines/{}/builds/{}"
        )

    def list_all(
        self,
        creator=None,
        created_from=None,
        created_to=None,
        finished_from=None,
        states=[],
        meta_data=None,
        branch=None,
        commit=None,
        include_retried_jobs=None,
        page=0,
        with_pagination=False,
    ):
        """
        Returns a paginated list of all builds across all the user's organizations and pipelines. If using
        token-based authentication the list of builds will be for the authorized organizations only. Builds are
        listed in the order they were created (newest first).

        :param creator: Filters the results by the user who created the build
        :param created_from: Filters the results by builds created on or after the given datetime.date
        :param created_to: Filters the results by builds created before the given datetime.date
        :param finished_from: Filters the results by builds finished on or after the given datetime.date
        :param states: Filters the results by build states [List]
        :param meta_data: Filters the results by the given meta_data. Example: ?meta_data[some-key]=some-value
        :param branch: Filters the results by the given branch or branches.
        :param commit: Filters the results by the commit (only works for full sha, not for shortened ones).
        :param include_retried_jobs: Include all retried job executions in each build's jobs list if True.
               Without this parameter, you'll see only the most recently run job for each step.
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of all builds across all the user's organizations and pipelines
        """
        self.__validate_dates([created_from, created_to, finished_from])
        self.__are_valid_states(states)

        query_params = {
            "creator": creator,
            "created_from": self.__api_date_format(created_from),
            "created_to": self.__api_date_format(created_to),
            "finished_from": self.__api_date_format(finished_from),
            "state": self.__get_build_states_query_param(states),
            "branch": self.__get_branches_query_param(branch),
            "commit": commit,
            "include_retried_jobs": True if include_retried_jobs is True else None,
            "page": page,
        }
        query_params.update(self.__process_meta_data(meta_data))

        return self.client.get(
            self.path_for_all, query_params, with_pagination=with_pagination
        )

    def list_all_for_org(
        self,
        organization,
        creator=None,
        created_from=None,
        created_to=None,
        finished_from=None,
        states=[],
        meta_data=None,
        branch=None,
        commit=None,
        include_retried_jobs=None,
        page=0,
        with_pagination=False,
    ):
        """
        Returns a paginated list of an organization's builds across all of an organization's pipelines. Builds are
        listed in the order they were created (newest first).

        :param organization: Organization slug
        :param creator: Filters the results by the user who created the build
        :param created_from: Filters the results by builds created on or after the given datetime.date
        :param created_to: Filters the results by builds created before the given datetime.date
        :param finished_from: Filters the results by builds finished on or after the given datetime.date
        :param states: Filters the results by build states [List]
        :param meta_data: Filters the results by the given meta_data.
        :param branch: Filters the results by the given branch or branches.
        :param commit: Filters the results by the commit (only works for full sha, not for shortened ones).
        :param include_retried_jobs: Include all retried job executions in each build's jobs list if True.
               Without this parameter, you'll see only the most recently run job for each step.
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of an organization's builds across all of an organization's pipelines.
        """

        self.__validate_dates([created_from, created_to, finished_from])
        self.__are_valid_states(states)

        query_params = {
            "creator": creator,
            "created_from": self.__api_date_format(created_from),
            "created_to": self.__api_date_format(created_to),
            "finished_from": self.__api_date_format(finished_from),
            "state": self.__get_build_states_query_param(states),
            "branch": self.__get_branches_query_param(branch),
            "commit": commit,
            "include_retried_jobs": True if include_retried_jobs is True else None,
            "page": page,
        }
        query_params.update(self.__process_meta_data(meta_data))

        return self.client.get(
            self.path_by_org.format(organization),
            query_params,
            with_pagination=with_pagination,
        )

    def list_all_for_pipeline(
        self,
        organization,
        pipeline,
        creator=None,
        created_from=None,
        created_to=None,
        finished_from=None,
        states=[],
        meta_data=None,
        branch=None,
        commit=None,
        include_retried_jobs=None,
        page=0,
        with_pagination=False,
    ):
        """
        Returns a paginated list of a pipeline's builds. Builds are listed in the order they were created (newest
        first).

        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param creator: Filters the results by the user who created the build
        :param created_from: Filters the results by builds created on or after the given datetime.date
        :param created_to: Filters the results by builds created before the given datetime.date
        :param finished_from: Filters the results by builds finished on or after the given datetime.date
        :param states: Filters the results by build states [List]
        :param meta_data: Filters the results by the given meta_data.
        :param branch: Filters the results by the given branch or branches.
        :param commit: Filters the results by the commit (only works for full sha, not for shortened ones).
        :param include_retried_jobs: Include all retried job executions in each build's jobs list if True.
               Without this parameter, you'll see only the most recently run job for each step.
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of a pipeline's builds.
        """

        self.__validate_dates([created_from, created_to, finished_from])
        self.__are_valid_states(states)

        query_params = {
            "creator": creator,
            "created_from": self.__api_date_format(created_from),
            "created_to": self.__api_date_format(created_to),
            "finished_from": self.__api_date_format(finished_from),
            "state": self.__get_build_states_query_param(states),
            "branch": self.__get_branches_query_param(branch),
            "commit": commit,
            "include_retried_jobs": True if include_retried_jobs is True else None,
            "page": page,
        }
        query_params.update(self.__process_meta_data(meta_data))

        return self.client.get(
            self.path_by_pipeline.format(organization, pipeline),
            query_params,
            with_pagination=with_pagination,
        )

    def get_build_by_number(
        self, organization, pipeline, build_number, include_retried_jobs=None
    ):
        """
        Get build by build number

        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param build_number: Build number
        :param include_retried_jobs: Include all retried job executions in each build's jobs list if True.
               Without this parameter, you'll see only the most recently run job for each step.
        :return: A build
        """
        query_params = {
            "include_retried_jobs": True if include_retried_jobs is True else None
        }
        return self.client.get(
            self.path_for_build_number.format(organization, pipeline, build_number),
            query_params=query_params,
        )

    def create_build(
        self,
        organization,
        pipeline,
        commit,
        branch,
        author=None,
        clean_checkout=None,
        env=None,
        ignore_pipeline_branch_filters=None,
        message=None,
        meta_data=None,
        pull_request_base_branch=None,
        pull_request_id=None,
        pull_request_repository=None,
        pull_request_labels=None,
    ):
        """
        Create a build

        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param commit: Commit to build
        :param branch: Branch to build
        :param author: Author of the build
        :param clean_checkout: Boolean to perform a clean checkout
        :param env: Any ENV variables for the build
        :param ignore_pipeline_branch_filters: Boolean to Ignore any branch filtering
        :param message: Message of the build
        :param meta_data: Meta Data for the build
        :param pull_request_base_branch: Base branch of a PR build
        :param pull_request_id: ID for a PR build
        :param pull_request_repository: Repository for a PR build
        :param pull_request_labels: List of labels assigned to the pull request
        :return: The created build
        """
        body = {
            "commit": commit,
            "branch": branch,
            "author": author,
            "clean_checkout": clean_checkout,
            "env": env,
            "ignore_pipeline_branch_filters": ignore_pipeline_branch_filters,
            "message": message,
            "meta_data": meta_data,
            "pull_request_base_branch": pull_request_base_branch,
            "pull_request_id": pull_request_id,
            "pull_request_repository": pull_request_repository,
            "pull_request_labels": pull_request_labels,
        }
        return self.client.post(
            self.path_by_pipeline.format(organization, pipeline), body
        )

    def cancel_build(self, organization, pipeline, build_number):
        cancel = "/cancel"
        return self.client.put(
            self.path_for_build_number.format(organization, pipeline, build_number)
            + cancel
        )

    def rebuild_build(self, organization, pipeline, build_number):
        rebuild = "/rebuild"
        return self.client.put(
            self.path_for_build_number.format(organization, pipeline, build_number)
            + rebuild
        )

    @staticmethod
    def __process_meta_data(meta_data):
        if not meta_data:
            return {}
        else:
            return {
                "meta_data[{}]".format(name): value for name, value in meta_data.items()
            }

    @staticmethod
    def __validate_dates(datetimes):
        for date in datetimes:
            if date is not None:
                if not isinstance(date, datetime.date):
                    raise NotValidDateTime

    @staticmethod
    def __api_date_format(datetime):
        if datetime is None:
            return None
        return datetime.isoformat()

    @staticmethod
    def __are_valid_states(states):
        if not isinstance(states, List):
            raise BuildStateNotAList
        if not states:
            return
        for state in states:
            if not isinstance(state, BuildState):
                raise NotValidBuildState

    @staticmethod
    def __get_build_states_query_param(states):
        if not states:
            return None
        if len(states) == 1:
            return "state=" + states[0].value
        else:
            param_string = ""
            for state in states:
                param_string += "state[]={}&".format(state.value)
            return param_string[:-1]

    @staticmethod
    def __get_branches_query_param(branches):
        if not branches:
            return None

        if isinstance(branches, List):
            param_string = ""
            for branch in branches:
                encoded_branch = urllib.parse.quote_plus(branch)
                param_string += f"branch[]={{}}&".format(encoded_branch)
            return param_string[:-1]
        else:
            encoded_branch = urllib.parse.quote_plus(branches)
            return f"branch={{}}".format(encoded_branch)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pybuildkite",
    version="1.3.0",
    url="https://github.com/pyasi/pybuildkite",
    download_url="https://github.com/pyasi/pybuildkite/archive/master.zip",
    author="Peter Yasi",
    packages=["pybuildkite"],
    description="Python wrapper for the Buildkite API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=["Buildkite", "Continuous Integration", "API", "CI", "wrapper", "python"],
    install_requires=["requests"],
)



from posixpath import join as urljoin

from pybuildkite.client import Client


class Meta(Client):
    """
    Meta operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = urljoin(base_url, "meta")

    def get_meta_information(self):
        """
        Returns meta information

        :return: Returns meta information
        """
        return self.client.get(self.path)



from pybuildkite.client import Client


class Organizations(Client):
    """
    Organization operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = base_url + "organizations/"

    def list_all(self, page=0, with_pagination=False):
        """
        Returns a paginated list of the user’s organizations.

        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Paginated list of the user’s organizations.
        """
        query_params = {"page": page}
        return self.client.get(
            self.path, query_params=query_params, with_pagination=with_pagination
        )

    def get_org(self, org_name):
        """
        Get an organization

        :param org_name: Organisation slug
        :return: Organisation
        """
        return self.client.get(self.path + org_name)



from pybuildkite.client import Client


class Users(Client):
    """
    User operations for the Buildkite API
    """

    def __init__(self, client: Client, base_url: str) -> None:
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = base_url + "user"

    def get_current_user(self):
        """
        Returns the current user

        :return: Returns current user
        """
        return self.client.get(self.path)



import pytest

from pybuildkite.artifacts import Artifacts


def test_list_artifacts_for_build(fake_client):
    """
    Test List Artifacts for build
    """
    artifacts = Artifacts(fake_client, "base")
    artifacts.list_artifacts_for_build("org_slug", "pipe_slug", "build_no")
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/artifacts/"
    fake_client.get.assert_called_with(
        url, query_params={"page": 0}, with_pagination=False
    )


def test_list_artifacts_for_job(fake_client):
    """
    Test list artifacts for job
    """
    artifacts = Artifacts(fake_client, "base")
    artifacts.list_artifacts_for_job("org_slug", "pipe_slug", "build_no", 123)
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/artifacts/"
    fake_client.get.assert_called_with(
        url, query_params={"page": 0}, with_pagination=False
    )


def test_get_artifact(fake_client):
    """
    Test get Artifact
    """
    artifacts = Artifacts(fake_client, "base")
    artifacts.get_artifact("org_slug", "pipe_slug", "build_no", 123, "artifact")
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/artifacts/artifact/"
    fake_client.get.assert_called_with(url)


def test_download_artifact(fake_client):
    """
    Test download Artifact
    """
    artifacts = Artifacts(fake_client, "base")
    artifacts.download_artifact("org_slug", "pipe_slug", "build_no", 123, "artifact")
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/artifacts/artifact/download/"
    fake_client.get.assert_called_with(
        url, headers={"Accept": "application/octet-stream"}, as_stream=False
    )


def test_download_artifact_as_stream(fake_client):
    """
    Test download Artifact as stream
    """
    artifacts = Artifacts(fake_client, "base")
    artifacts.download_artifact(
        "org_slug", "pipe_slug", "build_no", 123, "artifact", as_stream=True
    )
    url = "base/organizations/org_slug/pipelines/pipe_slug/builds/build_no/jobs/123/artifacts/artifact/download/"
    fake_client.get.assert_called_with(
        url, headers={"Accept": "application/octet-stream"}, as_stream=True
    )



from pybuildkite.client import Client


class Emojis(Client):
    """
    Emoji operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = base_url + "organizations/{}/emojis/"

    def list_emojis(self, organization, page=0, with_pagination=False):
        """
        Returns a list of all the emojis for a given organization

        :param organization: organization slug
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a list of all the emojis for a given organization
        """
        query_parms = {"page": page}
        return self.client.get(
            self.path.format(organization),
            query_params=query_parms,
            with_pagination=with_pagination,
        )



from pybuildkite.client import Client


class Pipelines(Client):
    """
    Pipeline operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client:
        :param base_url:
        """
        self.client = client
        self.path = base_url + "organizations/{}/pipelines/"

    def list_pipelines(self, organization, page=0, with_pagination=False):
        """
        Returns a paginated list of an organization’s pipelines.

        :param organization: Organization slug
        :param page: Int to determine which page to read from (See Pagination in README)
        :param with_pagination: Bool to return a response with pagination attributes
        :return: Returns a paginated list of an organization’s pipelines.
        """
        query_params = {"page": page}
        return self.client.get(
            self.path.format(organization),
            query_params=query_params,
            with_pagination=with_pagination,
        )

    def get_pipeline(self, organization, pipeline_name):
        """
        Get a pipeline

        :param organization: Organization slug
        :param pipeline_name: Pipeline slug
        :return: A pipeline
        """
        return self.client.get(self.path.format(organization) + pipeline_name)

    def create_pipeline(
        self,
        organization,
        pipeline_name,
        git_repository,
        build_steps=[
            dict(
                type="script",
                name=":pipeline:",
                command="buildkite-agent pipeline upload",
            )
        ],
        team_uuids: list = None,
        branch_configuration: str = None,
        default_branch: str = None,
        provider_settings: dict = None,
    ):
        """
        Create a pipeline for organizations using Web Visual Steps.
        See `create_yaml_pipeline` if you've migrated to YAML pipelines.
        :param build_steps: list of build pipeline steps
        Command: { "type": "script", "name": "Script", "command": "command.sh" }
        Wait for all previous steps to finish: { "type": "waiter" }
        Block pipeline (see the job unblock API): { "type": "manual" }
        Trigger pipeline: { "type": "trigger", "trigger_project_slug": "a-pipeline" }
        :param organization: Organization slug
        :param pipeline_name:Pipeline slug
        :param git_repository: repo URL
        :param team_uuids: A list of team_uuids. This property is only available if your organization has enabled Teams.
        :param branch_configuration: A branch filter pattern to limit which pushed branches or tags trigger builds on this pipeline.
        :param default_branch: The name of the default branch in the repository you've specified.
        :param provider_settings: A dictionary of provider-specific settings to configure.
        :return:
        """
        data = {
            "name": pipeline_name,
            "repository": git_repository,
            "steps": build_steps,
            "team_uuids": team_uuids,
            "branch_configuration": branch_configuration,
            "default_branch": default_branch,
            "provider_settings": provider_settings,
        }

        return self.client.post(self.path.format(organization), body=data)

    def create_yaml_pipeline(
        self,
        organization,
        pipeline_name,
        git_repository,
        configuration,
        team_uuids: list = None,
        branch_configuration: str = None,
        default_branch: str = None,
        provider_settings: dict = None,
    ):
        """
        Create a pipeline for organizations who have migrated to YAML pipelines
        https://buildkite.com/changelog/99-introducing-the-yaml-steps-editor
        :param organization: Organization slug
        :param pipeline_name:Pipeline slug
        :param git_repository: repo URL
        :param configuration: a valid pipeline.yml
        :param team_uuids: A list of team_uuids. This property is only available if your organization has enabled Teams.
        :param branch_configuration: A branch filter pattern to limit which pushed branches or tags trigger builds on this pipeline.
        :param default_branch: The name of the default branch in the repository you've specified.
        :param provider_settings: A dictionary of provider-specific settings to configure.
        :return:
        """
        data = {
            "name": pipeline_name,
            "repository": git_repository,
            "configuration": configuration,
            "team_uuids": team_uuids,
            "branch_configuration": branch_configuration,
            "default_branch": default_branch,
            "provider_settings": provider_settings,
        }
        return self.client.post(self.path.format(organization), body=data)

    def delete_pipeline(self, organization, pipeline):
        """
        Delete a pipeline
        :param organization: Organization slug
        :param pipeline:Pipeline slug
        :return:
        """
        url = self.path.format(organization) + pipeline
        return self.client.delete(url)

    def update_pipeline(
        self,
        organization,
        pipeline,
        branch_configuration: str = None,
        cancel_running_branch_builds: bool = None,
        cancel_running_branch_builds_filter: str = None,
        default_branch: str = None,
        description: str = None,
        env: dict = None,
        name: str = None,
        provider_settings: dict = None,
        repository: str = None,
        configuration: str = None,
        steps: dict = None,
        skip_queued_branch_builds: bool = None,
        skip_queued_branch_builds_filter: str = None,
        visibility: str = None,
    ):
        """
        Patch a pipeline.
        See https://buildkite.com/docs/apis/rest-api/pipelines#update-a-pipeline
        for documentation on each input

        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :return: Pipeline
        """
        if configuration is not None and steps is not None:
            raise PipelineException(
                "Cannot set both `configuration` and `steps`. If you've migrated to YAML steps, please use `configuration`. Otherwise, use `steps`"
            )

        body = {
            "branch_configuration": branch_configuration,
            "cancel_running_branch_builds": cancel_running_branch_builds,
            "cancel_running_branch_builds_filter": cancel_running_branch_builds_filter,
            "default_branch": default_branch,
            "description": description,
            "env": env,
            "name": name,
            "provider_settings": provider_settings,
            "repository": repository,
            "configuration": configuration,
            "steps": steps,
            "skip_queued_branch_builds": skip_queued_branch_builds,
            "skip_queued_branch_builds_filter": skip_queued_branch_builds_filter,
            "visibility": visibility,
        }
        url = self.path.format(organization) + pipeline
        return self.client.patch(url, body=body)

    def archive_pipeline(self, organization, pipeline):
        """
        Archive a pipeline
        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :return:
        """
        url = self.path.format(organization) + pipeline + "/archive"
        return self.client.post(url)

    def unarchive_pipeline(self, organization, pipeline):
        """
        Unarchive a pipeline
        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :return:
        """
        url = self.path.format(organization) + pipeline + "/unarchive"
        return self.client.post(url)


class PipelineException(Exception):
    pass



import pytest

from pybuildkite.agents import Agents


def test_get_agent(fake_client):
    """
    Test the get_agent method
    """
    agents = Agents(fake_client, "base")
    agents.get_agent("org_slug", "agent_id")
    fake_client.get.assert_called_with(agents.path.format("org_slug") + "agent_id")


def test_stop_agent(fake_client):
    agents = Agents(fake_client, "base")
    agents.stop_agent("org_slug", "agent_id")
    fake_client.put.assert_called_with(
        agents.path.format("org_slug") + "agent_id/stop", body={"force": True}
    )


def test_list_all_agents(fake_client):
    agents = Agents(fake_client, "base")
    agents.list_all("org_slug")
    fake_client.get.assert_called_with(
        agents.path.format("org_slug"),
        {"name": None, "hostname": None, "version": None, "page": 0},
        with_pagination=False,
    )



from enum import Enum
from posixpath import join as urljoin

from pybuildkite.client import Client


class LogFormat(Enum):
    """
    Valid log formats
    """

    TEXT = "text/plain"
    HTML = "text/html"
    JSON = "application/json"

    def __str__(self):
        return self.value


class Jobs(Client):
    """
    Job operations for the Buildkite API
    """

    def __init__(self, client, base_url):
        """
        Construct the class

        :param client: API Client
        :param base_url: Base Url
        """
        self.client = client
        self.path = urljoin(base_url, "organizations/{}/pipelines/{}/builds/{}/jobs/{}")

    def get_job_log(
        self, organization, pipeline, build, job, log_format=LogFormat.HTML
    ):
        """
        Get a job’s log output

        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param build: Build number
        :param job: Job id
        :param log_format: Mime type to return log in
        :return: Job log output
        """
        header = {"Accept": str(log_format)}
        return self.client.get(
            self.path.format(organization, pipeline, build, job) + "/log",
            headers=header,
        )

    def get_job_environment_variables(self, organization, pipeline, build, job):
        """
        Get a job's environment variables

        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param build: Build number
        :param job: Job id
        :return: Environment variables
        """
        return self.client.get(
            self.path.format(organization, pipeline, build, job) + "/env"
        )

    def retry_job(self, organization, pipeline, build, job):
        """
        Retries a failed or timed_out job.
         :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param build: Build number
        :param job: Job id
        :return: response
        """
        retry = "/retry"
        return self.client.put(
            self.path.format(organization, pipeline, build, job) + retry
        )

    def unblock_job(self, organization, pipeline, build, job, fields, unblocker=None):
        """
        Unblocks a build’s "Block pipeline" job.
        :param organization: Organization slug
        :param pipeline: Pipeline slug
        :param build: Build number
        :param job: Job id
        :fields: name and email
        :unblocker: The user id of the person activating the job
        :return: response
        """
        unblock = "/unblock"
        body = {"fields": fields, "unblocker": unblocker}
        return self.client.put(
            self.path.format(organization, pipeline, build, job) + unblock, body=body
        )

    def delete_job_log(self, organization, pipeline, build, job):
        """
        Delete a job’s log output
        :param organization: organization slug
        :param pipeline: pipeline slug
        :param build: Build number
        :param job: job id
        :return: success response 204 No content
        """
        log = "/log"
        return self.client.delete(
            self.path.format(organization, pipeline, build, job) + log
        )
